#ifndef FEBSIMULATOR_H
#define FEBSIMULATOR_H

#include "DAQDataFormats/RawNanoSlice.h"	/* Use the nanoslice data format */

namespace daqdataformats {


class FEBSimulator {

 public:

  FEBSimulator();
  ~FEBSimulator();
  FEBSimulator(uint32_t linkID);
  FEBSimulator(uint32_t linkID, float probability);

  void  advanceSEQ();		/* Advance the Sequence Number */
  void  resetSEQ();		/* Reset the Sequence Number */

  void setLinkID(uint32_t linkID);
  void setHitProbability(float probability);

  //! Generator methods produce a NanoSlice
  void generateNanoSlice();
  void generateNanoSlice(int PixelID);
  void generateNanoSlice(int PixelID, bool ForceGeneration);
  void generateNanoSlice(int PixelID, int TimeStamp, int PulseHeight, bool ForceGeneration);

  bool nonEmptyNanoSlice();
  
  RawNanoSlice* getLastNanoSlice();


 private:
  uint32_t  fSEQ;		 /* Frame Sequence Number 3 bits */
  uint32_t  fFEBLinkID;	 /* FEB Link Identifier 5 bits */
  float         fHitProbability; /*Probability of Generating a hit */

  RawNanoSlice* fLastNanoSlice;  /* Last Generated Nanoslice */
};

} /* END NAMESPACE daqdataformats */
#endif /* FEBSIMULATOR_H */
