#ifndef RAWCONFIGURATIONNAMESPACE_H
#define RAWCONFIGURATIONNAMESPACE_H

#include "DAQDataFormats/RawDAQData.h"

#include <stdarg.h>		// va_start

#include <vector>

namespace daqdataformats {

    #define DEFAULT_RAWCONFIGURATIONNAMESPACE_SIZE 1000 /* Maximum MicroBlock Buffer Size 64k */



    namespace rawconftail {

        enum ConfNamespaceMASKS{
            MASK_Char1 = 0xFF000000, //! Character 1 (8 bits)
            MASK_Char2 = 0x00FF0000, //! Character 1 (8 bits)
            MASK_Char3 = 0x0000FF00, //! Character 1 (8 bits)
            MASK_Char4 = 0x000000FF, //! Character 1 (8 bits)
        };

        enum ConfNamespaceSHIFTS{
            SHIFT_Char1 = 24, //! Character 1 (8 bits)
            SHIFT_Char2 = 16, //! Character 1 (8 bits)
            SHIFT_Char3 = 8,  //! Character 1 (8 bits)
            SHIFT_Char4 = 0,  //! Character 1 (8 bits)
        };

    }// end of namespace rawconfhdr



    class RawConfigurationNamespace : public RawDAQData {
      
    public:
      
      /// Constructors
      RawConfigurationNamespace (uint32_t tlvl=0); //! The default constructor
      RawConfigurationNamespace (const RawConfigurationNamespace& copy_in);  //! Deep copy constructor

      /// Destructor
      ~RawConfigurationNamespace();

      /// Initialization of the data members
      void init();
      
      /// Deep Copy
      void  copy(const RawConfigurationNamespace& copy_in);

      /// Setters
      bool setNamespace( std::string& ); //returns the sizer in words
      /// Setters for Chars
      bool setChar1(uint32_t letter, uint32_t word_number);
      bool setChar2(uint32_t letter, uint32_t word_number);
      bool setChar3(uint32_t letter, uint32_t word_number);
      bool setChar4(uint32_t letter, uint32_t word_number);


      /// Getters
      uint32_t getNamespace( std::string& ) const ;//returns the sizer in words
      /// Getters for Chars
      uint32_t getChar1(uint32_t word_number) const ;
      uint32_t getChar2(uint32_t word_number) const ;
      uint32_t getChar3(uint32_t word_number) const ;
      uint32_t getChar4(uint32_t word_number) const ;




      /// Reading method to read the whole datablock
      void* readData(void* datablock_buffer);
      // Suplementary reading methods that needs to be defined since readData is overloaded
      void* readData(void* buffer, uint32_t size);
      void* readData(void* buffer, uint32_t size, uint32_t _Buffer_start_position);


      /// Print the DataBlock structure
      void print() const;

      /// Size of the Data Block in words
      uint32_t sizeofdata() const;

    private:

      size_t                _fd_read_bytes;

      uint32_t              _tlvl; // debug "trace level mask"

      /// The actual internal buffer (not a pointer)
      std::vector<uint32_t> _InternalBuffer;

      void tprintf( unsigned lvl, const char *fmt, ... )
	__attribute__((format(printf, 3, 4))); // g++ note: attributes are not
               			          //  allowed on a function-definition
    };// end of class definition
} // END NAMESPCE daqdataformats
#endif /* RAWCONFIGURATIONNAMESPACE_H */
