#ifndef RAWCONFIGURATIONSYSTEMID_H
#define RAWCONFIGURATIONSYSTEMID_H

#include "DAQDataFormats/RawDAQData.h"

#include <vector>


namespace daqdataformats {

#define DEFAULT_CONFIGURATIONSYSTEMID_SIZE 1000 /* Maximum MilliSlice Buffer Size  */
#define SYSTEM_SIZE 1 // size of the block for the subsystem in the configuration header



    namespace rawconfsysid {

        enum SysIDMASKS{
            MASK_DAQSystemVersion     = 0x0000FFFF, //! Version number for the subsystem (first of them) (16 bits)
            MASK_DAQSystemID          = 0xFFFF0000, //! Identification of the subsystem  (first of them) (16 bits)
        };

        enum SysIDSHIFTS{
            SHIFT_DAQSystemVersion     = 0,  //! Version number for the subsystem (first of them)
            SHIFT_DAQSystemID          = 16, //! Identification of the subsystem  (first of them)
        };

    }// end of namespace rawconfhdr


  class RawConfigurationSystemID : public RawDAQData {
    
  public:
    // Constructors
    RawConfigurationSystemID (uint32_t buffersize = DEFAULT_CONFIGURATIONSYSTEMID_SIZE); //! The default constructor
    RawConfigurationSystemID (const RawConfigurationSystemID& copy_in);                  //! Deep copy constructor

    /// Destructor
    ~RawConfigurationSystemID();

    /// Initialization of Data members
    void init();
    
    /// Deep Copy
    void copy(const RawConfigurationSystemID& copy_in);

    /// Method to close the Configuration Header. Sets the lower markers, the CRC.
    bool close();


    /// Read method is for reading the whole RawConfigurationSystemID
    void* readData(void* configurationheader_buffer);
    // Suplementary reading methods
    void* readData(void* buffer, uint32_t size);
    void* readData(void* buffer, uint32_t size, uint32_t _Buffer_start_position);

    /// Size of MilliSlice _Buffer in words
    uint32_t sizeofdata() const;

    /// Print the MilliSlice structure
    void print() const;


    // Getters
    uint32_t getDAQSystemVersion(uint32_t isystem) const; // The count of the system starts from 1.
    uint32_t getDAQSystemID(uint32_t isystem)      const; // The count of the system starts from 1.
    uint32_t getFreeMemory()                       const; //! Free memory left in the buffer.
    uint32_t getSystemNumber() const; // Number of systems
    
    // Setters
    bool setDAQSystemVersion           (uint32_t isystem, uint32_t version); // The count of the system starts from 1.
    bool setDAQSystemID                (uint32_t isystem, uint32_t id);      // The count of the system starts from 1.
    bool setSystemNumber(uint32_t);
    /// Add a DAQ system to the Configuration Header
    bool addSystem(uint32_t sys_id, uint32_t sys_version);

  private:

    // The actual internal buffer (not a pointer).
    // Used only if the constructors with the default size are used
    std::vector <uint32_t>  _InternalBuffer;


    uint32_t _SystemNumber;


  };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWCONFIGURATIONSYSTEMID_H */
