#ifndef RAWMICROSLICEHEADER_H
#define RAWMICROSLICEHEADER_H

#include "DAQDataFormats/RawDAQData.h"


#define RAWMICROSLICEHEADER_SIZE 1

namespace daqdataformats {


    namespace rawmuhdr {

        enum MicroSliceHeaderMASKS{
            MASK_DP                = 0x80000000,
            MASK_SIM               = 0x40000000,
            MASK_LeadingByteCount  = 0x0000FFFF
        };

        enum MicroSliceHeaderSHIFTS{
            SHIFT_DP               = 31,
            SHIFT_SIM              = 30,
            SHIFT_LeadingByteCount = 0
        };

    };// end of namespace rawmuhdr

  
  class RawMicroSliceHeader: public RawDAQData {
    
  public:
    
    // Constructors
    RawMicroSliceHeader();                              //! The default constructor
    RawMicroSliceHeader(const RawMicroSliceHeader& copy_in);  //! Deep copy constructor
    RawMicroSliceHeader(void* microsliceheader_buffer); //! Shallow copy constructor

    /// Destructor
    ~RawMicroSliceHeader();
    /// Initialize the header. Or reset it
    void init();

    /// Deep Copy
    void  copy(const RawMicroSliceHeader& copy_in);

    // Getters
    bool     getDataPresent()    const;
    bool     getMonteCarloFlag() const;
    uint32_t getByteCount()      const;

    // Setters
    bool setDataPresent   (bool);
    bool setMonteCarloFlag(bool);
    bool setByteCount     (uint32_t);
    bool addByteCount     (uint32_t nbytes);
    bool resetByteCount();

    bool setBufferSource (void* pointer); //!< Set The Buffer pointer to a given location. Don't use it if you are not sure
    bool resetBufferSource();             //!< Reset the class to use its internal buffer
    bool clear();                         //!< Clear the buffer
    bool reset();                         //!< Reset the class data
    bool isInternalBuffer() const;              //!< Is the buffer internal?

    /// Print the MicroSlice Header
    void print() const;

    /// MicroSlice header size in words
    uint32_t sizeofdata() const;


    
  private:

    /// The actual internal buffer (not a pointer)
    uint32_t  _InternalBuffer[RAWMICROSLICEHEADER_SIZE];

  };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWMICROSLICEHEADER_H */
