#ifndef RAWMILLISLICEHEADER_H
#define RAWMILLISLICEHEADER_H

#include "DAQDataFormats/RawDCMChan.h"


#define MILLISLICEHEADER_SIZE 9

namespace daqdataformats {



    namespace rawmillihdr {

        enum Mode_t{
            Mode_Keep_All_MicroSlices      = 0,
            Mode_Discard_Empty_MicroSlices = 1,
        };


        // MilliSlice header delimiter. [0] value in the header, [1] value in the tail
        static const uint32_t Marker_value[2] = {0x00007777, 0x00007778};

        enum MilliSliceHeaderMASKS{
            MASK_Marker                   = 0xFFFF0000, //! Marker Mask                 (16 bits)
            MASK_Version                  = 0x00000FFF, //! Version Mask                (12 bits)
            MASK_SliceSize                = 0xFFFFFFFF,  //! MilliSlice Size Mask       (32 bits)
            MASK_Partition                = 0xFC000000, //! Partition                   ( 6 bits)
            MASK_DestBuffID               = 0x03FF0000, //! Dest Buffer ID Mask         (10 bits)
            MASK_DCMChan                  = 0x0000FFFF,
            MASK_SliceSEQ                 = 0xFFFFFFFF, //! Slice Seq Mask              (32 bits)
            MASK_TimeStartLow             = 0xFFFFFFFF, //! Start Time Low Mask         (32 bits)
            MASK_TimeStartHigh            = 0x00FFFFFF, //! Start Time High Mask        (24 bits)
            MASK_TimeLength               = 0x03FFFFFF, //! MilliSlice Time Length Mask (26 bits)
            MASK_Complete                 = 0x04000000, //! Completetess bit Mask
            MASK_Mode                     = 0x08000000, //! DCM Mode Mask
            MASK_ConnectionInitialization = 0x10000000, //! Initialization Connection bit Mask
            MASK_CloseConnection          = 0x20000000, //! Close Connection bit
            MASK_NumberOfMicroSlices      = 0x003FFFFF, //! Number of MicroSlices Mask  (22 bits)
            MASK_FrameSEQ                 = 0xFFC00000, //! Frame Seq Mask              (10 bits)
            MASK_MilliSliceIndexNumber    = 0x0000FFFF, //! The number of MilliSlice index entries (16 bits)
            MASK_MicroSliceTimeLength     = 0x0FFF0000, //! The conf'd time length of MicroSlices, in usec. (12 bits)
            MASK_MonteCarloFlag           = 0x20000000, //! SIM flag                    (1 bit)
            MASK_CRCCalculationUsed       = 0x40000000, //! Is CRC calculation used?    (1 bit)
            MASK_MilliSliceIndexUsed      = 0x80000000, //! Do we EVER produce the MilliSlice index? (1 bit)
        };

        enum MilliSliceHeaderSHIFTS{
            SHIFT_Marker                   = 16,
            SHIFT_Version                  = 0,
            SHIFT_SliceSize                = 0,
            SHIFT_Partition                = 26,
            SHIFT_DestBuffID               = 16,
            SHIFT_DCMChan                  = 0,
            SHIFT_SliceSEQ                 = 0,
            SHIFT_TimeStartLow             = 0,
            SHIFT_TimeStartHigh            = 0,
            SHIFT_TimeLength               = 0,
            SHIFT_Complete                 = 26,
            SHIFT_Mode                     = 27,
            SHIFT_ConnectionInitialization = 28,
            SHIFT_CloseConnection          = 29,
            SHIFT_NumberOfMicroSlices      = 0,
            SHIFT_FrameSEQ                 = 22,
            SHIFT_MilliSliceIndexNumber    = 0,
            SHIFT_MicroSliceTimeLength     = 16,
            SHIFT_MonteCarloFlag           = 29,
            SHIFT_CRCCalculationUsed       = 30,
            SHIFT_MilliSliceIndexUsed      = 31,
        };

        enum MilliSliceHeaderWORDS{
            WORD_Marker                   = 0,
            WORD_Version                  = 0,
            WORD_SliceSize                = 1,
            WORD_Partition                = 2,
            WORD_DestBuffID               = 2,
            WORD_DCMChan                  = 2,
            WORD_SliceSEQ                 = 3,
            WORD_TimeStartLow             = 4,
            WORD_TimeStartHigh            = 5,
            WORD_TimeLength               = 6,
            WORD_Complete                 = 6,
            WORD_Mode                     = 6,
            WORD_ConnectionInitialization = 6,
            WORD_CloseConnection          = 6,
            WORD_NumberOfMicroSlices      = 7,
            WORD_FrameSEQ                 = 7,
            WORD_MilliSliceIndexNumber    = 8,
            WORD_MicroSliceTimeLength     = 8,
            WORD_MonteCarloFlag           = 8,
            WORD_CRCCalculationUsed       = 8,
            WORD_MilliSliceIndexUsed      = 8,
        };
    }// end of namespace rawmillihdr

  
  class RawMilliSliceHeader: public RawDAQData {
    
  public:
    
    // Constructors
    RawMilliSliceHeader();                              //! The default constructor
    RawMilliSliceHeader(const RawMilliSliceHeader& copy_in);  //! Deep copy constructor
    RawMilliSliceHeader(void* millisliceheader_buffer); //! Shallow copy constructor
    /// Destructor
    ~RawMilliSliceHeader();

    /// Initialization of the data members
    void init();

    /// Deep Copy
    void  copy(const RawMilliSliceHeader& copy_in);

    /// Print the MilliSlice Header
    void print() const;

    /// Getters
    RawDCMChan*           getDCMChan();
    uint32_t              getVersion()                 const;
    uint32_t              getMarker()                  const;
    uint32_t              getDestBuffID()              const;
    uint32_t              getPartition()               const;
    uint64_t              getTimeStart()               const;
    uint32_t              getTimeStartLow()            const;
    uint32_t              getTimeStartHigh()           const;
    uint32_t              getTimeLength()              const;
    bool                  isComplete()                 const;
    rawmillihdr::Mode_t   getMode()                    const;
    bool                  isConnectionInitialization() const;
    bool                  isCloseConnection()          const;
    uint32_t              getMilliSliceSEQ()           const;
    uint32_t              getFrameSEQ()                const;
    uint32_t              getSliceSize()               const;
    uint32_t              getMicrosliceNumber()        const;
    uint32_t              getMilliSliceIndexNumber()   const;
    uint32_t              getMicroSliceTimeLength_usec() const;
    bool                  getMonteCarloFlag()          const;
    bool                  isCRCCalculationUsed()       const;
    bool                  isMilliSliceIndexUsed()      const;
    uint32_t              getDiblockId()               const;
    uint32_t              getDCMId()                   const;
    uint32_t              getDetId()                   const;
    uint32_t              getGlobalDCMID()             const;

    // Setters
    bool setDiblockId                (uint32_t);
    bool setDCMId                    (uint32_t);
    bool setDetId                    (uint32_t);
    bool setGlobalDCMID              (uint32_t);
    bool setVersion                  (uint32_t);
    bool setMarker                   (uint32_t);
    bool setDestBuffID               (uint32_t);
    bool setPartition                (uint32_t);
    bool setTimeStart                (uint64_t);
    bool setTimeStartLow             (uint32_t);
    bool setTimeStartHigh            (uint32_t);
    bool setTimeLength               (uint32_t);
    bool setComplete                 (bool);
    bool setMode                     (rawmillihdr::Mode_t);
    bool setConnectionInitialization (bool);
    bool setCloseConnection          (bool);
    bool setMilliSliceSEQ            (uint32_t);
    bool setFrameSEQ                 (uint32_t);
    bool setSliceSize                (uint32_t);
    bool setMicrosliceNumber         (uint32_t);
    bool setMilliSliceIndexNumber    (uint32_t);
    bool setMicroSliceTimeLength_usec(uint32_t);
    bool setMonteCarloFlag           (bool);
    bool setCRCCalculationUsed       (bool);
    bool setMilliSliceIndexUsed      (bool);
    bool advanceMicrosliceNumber();
    bool advanceFrameSEQ();
    bool resetFrameSEQ();
    bool advanceMilliSliceSEQ();
    bool addSliceSize                (uint32_t size);

    bool setBufferSource (void* pointer); //!< Set The Buffer pointer to a given location. Don't use it if you are not sure
    bool resetBufferSource();             //!< Reset the class to use its internal buffer
    bool clear();                         //!< Clear the buffer
    bool reset();                         //!< Reset the class data
    bool isInternalBuffer() const;              //!< Is the buffer internal?


    /// MilliSlice Header size in words
    uint32_t sizeofdata() const;

  private:
    
    RawDCMChan   _DCMChan;      //! DCM identifier

    /// The actual internal buffer (not a pointer)
    uint32_t  _InternalBuffer[MILLISLICEHEADER_SIZE];
  };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWMILLISLICEHEADER_H */
