#include "DAQDataFormats/RawEventHeader.h"
#include <cstdio>

#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif

using namespace daqdataformats;
using namespace raweventhdr;

//------------------------------------------------------------
// Default constructor
RawEventHeader::RawEventHeader(){

    // Set the main buffer pointer to the internal buffer
    this->resetBufferSource();
    // Initialize it
    this->init();
}

//------------------------------------------------------------------------------
RawEventHeader::RawEventHeader(const RawEventHeader& copy_in){
    this->copy(copy_in);
}

//------------------------------------------------------------------------------
RawEventHeader::RawEventHeader(void* eventheader_buffer){
    // First, initialize the Internal Buffer
            // Set pointers to point to the internal buffer
            this->resetBufferSource();

            // Initialize it
            this->init();

    // Now set the pointer to the external buffer
    this->setBufferSource(eventheader_buffer);
}

//------------------------------------------------------------
// Destructor
RawEventHeader::~RawEventHeader() { }

//------------------------------------------------------------
// Initialize the data members
void RawEventHeader::init(){

    // Zero out the Buffer first
    for(uint32_t i = 0; i < this->sizeofdata(); i++)
            RAWBUFF32[i] = 0;

    // Setup the RawEnentHeader markers
    this->setMarkerLeft (Marker_value[0]);
    this->setMarkerRight(Marker_value[1]);
}
//------------------------------------------------------------------------------
void RawEventHeader::copy(const RawEventHeader& copy_in){
    // Copy the data from one buffer to the other
    for(uint32_t i = 0; i < this->sizeofdata(); ++i)
        _InternalBuffer[i] = copy_in._InternalBuffer[i];

    // If the copy_in's buffer is external, point to that external memory
    if(copy_in.isInternalBuffer()) this->resetBufferSource();
    else                           this->setBufferSource(copy_in.getBuffer());
}

//------------------------------------------------------------------------------
// Getters
    uint32_t RawEventHeader::getMarkerLeft()    const {return BITFIELDGET(RAWBUFF32[WORD_MarkerLeft]     , MASK_MarkerLeft     , SHIFT_MarkerLeft);}
    uint32_t RawEventHeader::getMarkerRight()   const {return BITFIELDGET(RAWBUFF32[WORD_MarkerRight]    , MASK_MarkerRight    , SHIFT_MarkerRight);}
    uint32_t RawEventHeader::getVersion()       const {return BITFIELDGET(RAWBUFF32[WORD_Version]        , MASK_Version        , SHIFT_Version);}
    uint32_t RawEventHeader::getDataLoggerID()  const {return BITFIELDGET(RAWBUFF32[WORD_DataLoggerID]   , MASK_DataLoggerID   , SHIFT_DataLoggerID);}
    uint32_t RawEventHeader::getRunNumber()     const {return BITFIELDGET(RAWBUFF32[WORD_RunNumber]      , MASK_RunNumber      , SHIFT_RunNumber);}
    uint32_t RawEventHeader::getRunType()       const {return BITFIELDGET(RAWBUFF32[WORD_RunType]        , MASK_RunType        , SHIFT_RunType);}
    uint32_t RawEventHeader::getConfigIDX()     const {return BITFIELDGET(RAWBUFF32[WORD_ConfigIDX]      , MASK_ConfigIDX      , SHIFT_ConfigIDX);}
    uint32_t RawEventHeader::getPartition()     const {return BITFIELDGET(RAWBUFF32[WORD_Partition]      , MASK_Partition      , SHIFT_Partition);}
    uint32_t RawEventHeader::getSubrun()        const {return BITFIELDGET(RAWBUFF32[WORD_Subrun]         , MASK_Subrun         , SHIFT_Subrun);}
    uint32_t RawEventHeader::getEventNumberLo() const {return BITFIELDGET(RAWBUFF32[WORD_EventNumberLo]  , MASK_EventNumberLo  , SHIFT_EventNumberLo);}
    uint32_t RawEventHeader::getEventNumberHi() const {return BITFIELDGET(RAWBUFF32[WORD_EventNumberHi]  , MASK_EventNumberHi  , SHIFT_EventNumberHi);}
    uint32_t RawEventHeader::getEventType()     const {return BITFIELDGET(RAWBUFF32[WORD_EventType]      , MASK_EventType      , SHIFT_EventType);}
    uint32_t RawEventHeader::getEventSize()     const {return BITFIELDGET(RAWBUFF32[WORD_EventSize]      , MASK_EventSize      , SHIFT_EventSize);}
    uint32_t RawEventHeader::getDataBlockNumber()const{return BITFIELDGET(RAWBUFF32[WORD_DataBlockNumber], MASK_DataBlockNumber, SHIFT_DataBlockNumber);}
    uint64_t RawEventHeader::getEventNumber() const {
          uint64_t event;
          uint32_t low, high;

          low  = this->getEventNumberLo();
          high = this->getEventNumberHi();

          event = (uint64_t)high<<32 | low;
          return event;
    }

    bool     RawEventHeader::isEventIncomplete() const {
        if(BITFIELDGET(RAWBUFF32[WORD_EventIncompleteFlag], MASK_EventIncompleteFlag, SHIFT_EventIncompleteFlag))
            return true;
        return false;
    }

    bool     RawEventHeader::getMonteCarloFlag() const {
        if(BITFIELDGET(RAWBUFF32[WORD_MonteCarloFlag]    , MASK_MonteCarloFlag    , SHIFT_MonteCarloFlag))
            return true;
        return false;
    }
    bool     RawEventHeader::isCRCCalculationUsed() const {
        if(BITFIELDGET(RAWBUFF32[WORD_CRCCalculationUsed], MASK_CRCCalculationUsed, SHIFT_CRCCalculationUsed))
            return true;
        return false;
    }






    bool RawEventHeader::setMarkerLeft      (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_MarkerLeft]     , inp, MASK_MarkerLeft     , SHIFT_MarkerLeft);
        return true;
    }
    bool RawEventHeader::setMarkerRight     (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_MarkerRight]    , inp, MASK_MarkerRight    , SHIFT_MarkerRight);
        return true;
    }
    bool RawEventHeader::setVersion         (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_Version]        , inp, MASK_Version        , SHIFT_Version);
        return true;
    }
    bool RawEventHeader::setDataLoggerID    (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_DataLoggerID]   , inp, MASK_DataLoggerID   , SHIFT_DataLoggerID);
        return true;
    }
    bool RawEventHeader::setRunNumber       (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_RunNumber]      , inp, MASK_RunNumber      , SHIFT_RunNumber);
        return true;
    }
    bool RawEventHeader::setRunType         (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_RunType]        , inp, MASK_RunType        , SHIFT_RunType);
        return true;
    }
    bool RawEventHeader::setConfigIDX       (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_ConfigIDX]      , inp, MASK_ConfigIDX      , SHIFT_ConfigIDX);
        return true;
    }
    bool RawEventHeader::setPartition       (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_Partition]      , inp, MASK_Partition      , SHIFT_Partition);
        return true;
    }
    bool RawEventHeader::setSubrun          (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_Subrun]         , inp, MASK_Subrun         , SHIFT_Subrun);
        return true;
    }
    bool RawEventHeader::setEventNumberLo   (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_EventNumberLo]  , inp, MASK_EventNumberLo  , SHIFT_EventNumberLo);
        return true;
    }
    bool RawEventHeader::setEventNumberHi   (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_EventNumberHi]  , inp, MASK_EventNumberHi  , SHIFT_EventNumberHi);
        return true;
    }
    bool RawEventHeader::setEventType       (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_EventType]      , inp, MASK_EventType      , SHIFT_EventType);
        return true;
    }
    bool RawEventHeader::setEventSize       (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_EventSize]      , inp, MASK_EventSize      , SHIFT_EventSize);
        return true;
    }

    bool RawEventHeader::setDataBlockNumber (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_DataBlockNumber], inp, MASK_DataBlockNumber, SHIFT_DataBlockNumber);
        return true;
    }


    bool RawEventHeader::setEventNumber(uint64_t event) {
          uint32_t low, high;

          low  = event & 0xffffffff;
          high = event >> 32;

          this->setEventNumberHi(high);
          this->setEventNumberLo(low);

          return true;
    }
    bool RawEventHeader::setEventIncomplete (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_EventIncompleteFlag], 0x1 , MASK_EventIncompleteFlag, SHIFT_EventIncompleteFlag);
        else  BITFIELDSET(RAWBUFF32[WORD_EventIncompleteFlag], 0x0 , MASK_EventIncompleteFlag, SHIFT_EventIncompleteFlag);
        return true;
    }

    bool RawEventHeader::setMonteCarloFlag (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_MonteCarloFlag]    , 0x1 , MASK_MonteCarloFlag    , SHIFT_MonteCarloFlag);
        else  BITFIELDSET(RAWBUFF32[WORD_MonteCarloFlag]    , 0x0 , MASK_MonteCarloFlag    , SHIFT_MonteCarloFlag);
        return true;
    }
    bool RawEventHeader::setCRCCalculationUsed (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_CRCCalculationUsed], 0x1 , MASK_CRCCalculationUsed, SHIFT_CRCCalculationUsed);
        else  BITFIELDSET(RAWBUFF32[WORD_CRCCalculationUsed], 0x0 , MASK_CRCCalculationUsed, SHIFT_CRCCalculationUsed);
        return true;
    }





    bool RawEventHeader::addEventSize (uint32_t size) {return this->setEventSize(this->getEventSize() + size);}
    bool RawEventHeader::advanceDataBlockNumber()     {return this->setDataBlockNumber(this->getDataBlockNumber() + 1);}
    bool RawEventHeader::advanceEventNumber ()        {return this->setEventNumber(this->getEventNumber() + 1);}

    bool RawEventHeader::setBufferSource(void* pointer){
        _Buffer = pointer;
        return true;
    }
    bool RawEventHeader::resetBufferSource(){
        this->setBufferSource(&_InternalBuffer[0]);
        return true;
    }

// clears all data in the buffer
    bool RawEventHeader::clear(){
        int max_data = this->sizeofdata();

        // Loop over the buffer and zero it
        for(int i=0; i< max_data; ++i)
            RAWBUFF32[i]=0;

        return true;
    }
    bool RawEventHeader::reset(){
        this->resetBufferSource();
        this->clear();
        return true;
    }
    bool RawEventHeader::isInternalBuffer() const {

        if((this->_Buffer) == (void*)(&_InternalBuffer[0]))
            return true;
        return false;
  }


//------------------------------------------------------------------------------
// Print the MilliSlice header information
void RawEventHeader::print() const{
    printf("\nRawEvent Header:");
    printf(" MarkerLeft=%u"          , this->getMarkerLeft());   //! MilliSlice header delimiter
    printf(" MarkerRight=%u"         , this->getMarkerRight());  //! MilliSlice header delimiter
    printf(" Version=%u"             , this->getVersion());      //! Version number for the MilliSlice event builder
    printf(" DataLoggerID=%u"        , this->getDataLoggerID());
    printf(" RunNumber=%u"           , this->getRunNumber());
    printf(" RunType=%u"             , this->getRunType());
    printf(" ConfigIDX=%u"           , this->getConfigIDX());
    printf(" Partition=%u"           , this->getPartition());
    printf(" Subrun=%u"              , this->getSubrun());
    printf(" EventNumberLo=%u"       , this->getEventNumberLo());
    printf(" EventNumberHi=%u"       , this->getEventNumberHi());
    printf(" EventType=%u"           , this->getEventType());
    printf(" EventSize=%u"           , this->getEventSize());
    printf(" DataBlockNumber=%u"     , this->getDataBlockNumber());
    printf(" isEventIncomplete=%u"   , this->isEventIncomplete());
    printf(" MonteCarloFlag=%u"      , this->getMonteCarloFlag());
    printf(" isCRCCalculationUsed=%u", this->isCRCCalculationUsed());
   
    std::cout<<"\nRAW:";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        printf(" ");
        this->printWord(i,false);
    }

    std::cout<<"\nRAW:";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        printf("\n");
        this->printWord(i,true);
    }

}

//------------------------------------------------------------------------------
// Size of _Buffer in words
uint32_t RawEventHeader::sizeofdata()const {return RAWEVENTHEADER_SIZE;}
