#include "DAQDataFormats/RawMicroBlockHeader.h"
#include <cstdio>

#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif


using namespace daqdataformats;
using namespace rawmicroblockhdr;

//------------------------------------------------------------
// Default constructor
RawMicroBlockHeader::RawMicroBlockHeader(){
    // Set the main buffer pointer to the internal buffer
    this->resetBufferSource();
    // Initialize it
    this->init();
}

//------------------------------------------------------------------------------
RawMicroBlockHeader::RawMicroBlockHeader(const RawMicroBlockHeader& copy_in){
    this->copy(copy_in);
}

//------------------------------------------------------------------------------
RawMicroBlockHeader::RawMicroBlockHeader(void* millisliceheader_buffer){
    // First, initialize the Internal Buffer
            // Set pointers to point to the internal buffer
            this->resetBufferSource();

            // Initialize it
            this->init();

    // Now set the pointer to the external buffer
    this->setBufferSource(millisliceheader_buffer);

}

//------------------------------------------------------------
// Destructor
RawMicroBlockHeader::~RawMicroBlockHeader() { }

//------------------------------------------------------------
// Initialize the data members
void RawMicroBlockHeader::init(){

    for(uint32_t i = 0; i < this->sizeofdata(); i++)
        RAWBUFF32[i] = 0;
}
//------------------------------------------------------------------------------
void RawMicroBlockHeader::copy(const RawMicroBlockHeader& copy_in){

    // Copy the data from one buffer to the other
    for(uint32_t i = 0; i < this->sizeofdata(); ++i)
        _InternalBuffer[i] = copy_in._InternalBuffer[i];

    // If the copy_in's buffer is external, point to that external memory
    if(copy_in.isInternalBuffer()) this->resetBufferSource();
    else                           this->setBufferSource(copy_in.getBuffer());
}

//------------------------------------------------------------------------------
// Getters

    RawDCMChan* RawMicroBlockHeader::getDCMChan()                     { return &_DCMChan; }
    uint32_t    RawMicroBlockHeader::getDestBuffID() const            { return BITFIELDGET(RAWBUFF32[WORD_DestBuffID]         , MASK_DestBuffID         , SHIFT_DestBuffID); }
    uint32_t    RawMicroBlockHeader::getPartition() const             { return BITFIELDGET(RAWBUFF32[WORD_Partition]          , MASK_Partition          , SHIFT_Partition); }
    uint32_t    RawMicroBlockHeader::getDiblockId() const  { return _DCMChan.getDiblockId();}
    uint32_t    RawMicroBlockHeader::getDCMId()  const     { return _DCMChan.getDCMId();}
    uint32_t    RawMicroBlockHeader::getDetId() const      { return _DCMChan.getDetId();}
    uint32_t    RawMicroBlockHeader::getGlobalDCMID() const{ return _DCMChan.getGlobalDCMID();}

//------------------------------------------------------------------------------
// Setters
    bool RawMicroBlockHeader::setDiblockId  (uint32_t id) { return _DCMChan.setDiblockId(id);}
    bool RawMicroBlockHeader::setDCMId      (uint32_t id) { return _DCMChan.setDCMId(id);}
    bool RawMicroBlockHeader::setDetId      (uint32_t id) { return _DCMChan.setDetId(id);}
    bool RawMicroBlockHeader::setGlobalDCMID(uint32_t id) { return _DCMChan.setGlobalDCMID(id);}
    bool RawMicroBlockHeader::setDestBuffID      (uint32_t b) {
        BITFIELDSET(RAWBUFF32[WORD_DestBuffID]         , b  , MASK_DestBuffID         , SHIFT_DestBuffID);
        return true;
    }
    bool RawMicroBlockHeader::setPartition       (uint32_t p) {
        BITFIELDSET(RAWBUFF32[WORD_Partition]          , p  , MASK_Partition          , SHIFT_Partition);
        return true;
    }

    bool RawMicroBlockHeader::setBufferSource(void* pointer){
        _Buffer = pointer;
        uint32_t* pp = (uint32_t*)pointer + WORD_DCMChan;
        _DCMChan.setBufferSource(pp);
        return true;
}
    bool RawMicroBlockHeader::resetBufferSource(){
        this->setBufferSource(&_InternalBuffer[0]);
        return true;
    }

// clears all data in the buffer
    bool RawMicroBlockHeader::clear(){
        int max_data = this->sizeofdata();

        // Loop over the buffer and zero it
        for(int i=0; i< max_data; ++i)
            RAWBUFF32[i]=0;

        return true;
    }
    bool RawMicroBlockHeader::reset(){
        this->resetBufferSource();
        this->clear();
        return true;
    }
    bool RawMicroBlockHeader::isInternalBuffer() const {

        if((this->_Buffer) == (void*)(&_InternalBuffer[0]))
            return true;
        return false;
  }


//------------------------------------------------------------------------------
// Print the MicroBlock header information
void RawMicroBlockHeader::print() const{
    printf("\nMicroBlock Header:");
    printf(" getDiblockId=%u" , this->getDiblockId());
    printf(" DCMId=%u"        , this->getDCMId());
    printf(" DetId=%u"        , this->getDetId());
    printf(" DestBuff=%u"     , this->getDestBuffID());   //! Destination buffer node
    printf(" Partition=%u"    , this->getPartition());    //! Partition

    std::cout<<"\nMicroBlockHeader RAW 32bit words (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        printf(" ");
        this->printWord(i,false);
    }

    std::cout<<"\nMicroBlockHeader RAW bits (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        printf("\n");
        this->printWord(i,true);
    }

}

//------------------------------------------------------------------------------
// Size of _Buffer in words
uint32_t RawMicroBlockHeader::sizeofdata()const {return RAWMICROBLOCKHEADER_SIZE;}
