  #include "DAQDataFormats/RawNanoSlice.h"
#include <cstdio>

#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif
  
  using namespace daqdataformats;
  using namespace rawnano;
 
//------------------------Constructors------------------------------------------
RawNanoSlice::RawNanoSlice(){
    // Set pointers to point to the internal buffer
    this->resetBufferSource();

    // Initialize it
    this->init();
}

//------------------------------------------------------------------------------
RawNanoSlice::RawNanoSlice (NanoSlice nanoslice_parameters){
    // Set pointers to point to the internal buffer
    this->resetBufferSource();

    // Initialize it
    this->initwithData(nanoslice_parameters);
}
//------------------------------------------------------------------------------
RawNanoSlice::RawNanoSlice(const RawNanoSlice& copy_in){
    this->copy(copy_in);
}

//------------------------------------------------------------------------------
RawNanoSlice::RawNanoSlice(void* nanoslice_buffer){
    // First, initialize the Internal Buffer
            // Set pointers to point to the internal buffer
            this->resetBufferSource();

            // Initialize it
            this->init();

    // Now set the pointer to the external buffer
    this->setBufferSource(nanoslice_buffer);
}

//------------------------Destructor--------------------------------------------
RawNanoSlice::~RawNanoSlice() { }


//------------------------------------------------------------------------------
// Default Initialization of Data members
void RawNanoSlice::init(){

      // Setup the header pointer
    _Header.init();

      // Zero out the rest of the NanoSlice
    for(uint32_t i = _Header.sizeofdata(); i < this->sizeofdata(); ++i)
            RAWBUFF32[i] = 0;


    //std::cout<<"Initializeing to"<<cafe_word<<"\n";

    /// Initialize the Reserved word
    this->setReserved(cafe_word);


}

void RawNanoSlice::initwithData(NanoSlice nanoslice_parameters){

      // Setup the header pointer
      _Header.initWithData(nanoslice_parameters.Header);

      // Zero out the rest of the NanoSlice
      for(uint32_t i = _Header.sizeofdata(); i < this->sizeofdata(); ++i)
            RAWBUFF32[i] = 0;

      // Set the TimeStamp and PulseHeight
      this->setTimeStamp(nanoslice_parameters.TimeStamp);
      this->setPulseHeight(nanoslice_parameters.PulseHeight);
      this->setReserved(nanoslice_parameters.Reserved);
}

//------------------------------------------------------------------------------
// Deep Copy Method
  void RawNanoSlice::copy(const RawNanoSlice& copy_in){
    // Copy the data from one buffer to the other
    for(int i = 0; i < RAWNANOSLICE_SIZE; ++i)
        _InternalBuffer[i] = copy_in._InternalBuffer[i];

    // Setup the main buffer pointer
    // If the copy_in's buffer is external, point to that external memory
    if(copy_in.isInternalBuffer()) this->resetBufferSource();
    else                           this->setBufferSource(copy_in._Buffer);
  }

//
// GET METHODS
//
// get the NanoSlice header's pointer
RawNanoSliceHeader* RawNanoSlice::getHeader()      { return &_Header;}
// get Method for Pulse Height
uint32_t            RawNanoSlice::getPulseHeight() const { return BITFIELDGET(RAWBUFF32[2], MASK_PulseHeight, SHIFT_PulseHeight);}
uint32_t            RawNanoSlice::getReserved()    const { return BITFIELDGET(RAWBUFF32[2], MASK_Reserved   , SHIFT_Reserved);}
// get Method for Time Stamp
uint32_t            RawNanoSlice::getTimeStamp()   const { return BITFIELDGET(RAWBUFF32[1], MASK_TimeStamp  , SHIFT_TimeStamp);}



//------------------------------------------------------------------------------
// Setters
// set Method for Time Stamp
bool RawNanoSlice::setTimeStamp(uint32_t timestamp){
    BITFIELDSET(RAWBUFF32[1],  timestamp  , MASK_TimeStamp  , SHIFT_TimeStamp);
    return true;
}

// set Method for Pulse height
bool RawNanoSlice::setPulseHeight(uint32_t pulseheight){
    BITFIELDSET(RAWBUFF32[2], pulseheight , MASK_PulseHeight, SHIFT_PulseHeight);
    return true;
}
// set Method for Pulse height
bool RawNanoSlice::setReserved(uint32_t reserved){
    BITFIELDSET(RAWBUFF32[2], reserved , MASK_Reserved, SHIFT_Reserved);
    return true;
}
// Set all parameters
bool RawNanoSlice::setParameters(NanoSlice nanoslice_parameters){
   this->setTimeStamp(nanoslice_parameters.TimeStamp);
   this->setPulseHeight(nanoslice_parameters.PulseHeight);
   return _Header.setParameters(nanoslice_parameters.Header);
}

bool RawNanoSlice::setBufferSource(void* pointer){
   _Buffer                = pointer;
   _Header.setBufferSource(pointer);
   return true;
}
    bool RawNanoSlice::resetBufferSource(){
        this->setBufferSource(&_InternalBuffer[0]);
        return true;
    }

// clears all data in the buffer
    bool RawNanoSlice::clear(){
        int max_data = this->sizeofdata();

        // Loop over the buffer and zero it
        for(int i=0; i< max_data; ++i)
            RAWBUFF32[i]=0;

        return true;
    }
    bool RawNanoSlice::reset(){
        this->resetBufferSource();
        this->clear();
        return true;
    }
    bool RawNanoSlice::isInternalBuffer() const {

        if((this->_Buffer) == (void*)(&_InternalBuffer[0]))
            return true;
        return false;
  }


bool RawNanoSlice::checkReservedForCafe(){

    if(this->getReserved() == cafe_word)return true;

    return false;
}




//------------------------------------------------------------------------------
// Printing the NanoSlice
void RawNanoSlice::print() const{

    _Header.print();
    printf(" NanoSlice Body: TimeStamp=%u PulseHeight=%u Reserved=%u",
                                                               this->getTimeStamp(),
                                                               this->getPulseHeight(),
                                                               this->getReserved()
          );

   std::cout<<"\nRAW(hex):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
            this->printWord(i,false);
            printf(" ");
            }


   std::cout<<"\nRAW(binary):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
            if(i > 0)std::cout<<"    ";
            this->printWord(i,true);
            std::cout<<"\n";
            }
}
// Method to return the size of the data block in words
uint32_t RawNanoSlice::sizeofdata()const{

  return RAWNANOSLICE_SIZE; //size in words
}
