#include <cppunit/extensions/HelperMacros.h>
#include <DAQDataFormats/DataFormatException.h>
#include <DAQDataFormats/RawTriggerHeader.h>
#include <cstring>

#define PATTERN1 0x01234567
#define PATTERN2 0x76543210
#define PATTERN3 0x89abcdef
#define PATTERN4 0xfedcba98
#define PATTERN5 0xf0f0f0f0
#define PATTERN6 0xe1e1e1e1
#define PATTERN7 0xd2d2d2d2
#define PATTERN8 0xc3c3c3c3
#define PATTERN9 0xb4b4b4b4
#define PATTERN0 0xa5a5a5a5

#define BUFFER_WORDS 1000
#define SMALL_BUFFER_WORDS 1

using namespace daqdataformats;

class RawTriggerHeaderTests : public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE(RawTriggerHeaderTests);
    CPPUNIT_TEST(testBasicSetAndGet);
    CPPUNIT_TEST(testSetAndGetWithExternalBuffer);
    CPPUNIT_TEST(testCopyConstructorWithReference);
    CPPUNIT_TEST(testConstructionFromExternalBuffer);
    CPPUNIT_TEST(testNoDeleteOfExternalBuffer);
    CPPUNIT_TEST(testNoModsToExternalBuffer);
    CPPUNIT_TEST(testSmallExternalBuffer);
    CPPUNIT_TEST(testExtBuffToExtBuffCopy);
    CPPUNIT_TEST(testInitializationOfDelimiter);
    CPPUNIT_TEST_SUITE_END();

public:
    void setUp();
    void tearDown();

    void testBasicSetAndGet();
    void testSetAndGetWithExternalBuffer();
    void testCopyConstructorWithReference();
    void testConstructionFromExternalBuffer();
    void testNoDeleteOfExternalBuffer();
    void testNoModsToExternalBuffer();
    void testSmallExternalBuffer();
    void testExtBuffToExtBuffCopy();
    void testInitializationOfDelimiter();

private:

};

void
RawTriggerHeaderTests::setUp()
{
    // flush stdout to get progress markers to display in realtime
    std::cout.flush();
}

void
RawTriggerHeaderTests::tearDown()
{
    // flush stdout to get progress markers to display in realtime
    std::cout.flush();
}

void
RawTriggerHeaderTests::testBasicSetAndGet()
{
    RawTriggerHeader triggerHeader;

    // test that we get the same values as what we set
    triggerHeader.setHWFlag(0);
    CPPUNIT_ASSERT(triggerHeader.getHWFlag() == 0);
    triggerHeader.setHWFlag(1);
    CPPUNIT_ASSERT(triggerHeader.getHWFlag() == 1);

    triggerHeader.setSource(TRIGSRC_BEAMSTAMP_SYSTEM);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_BEAMSTAMP_SYSTEM);
    triggerHeader.setSource(TRIGSRC_PULSER_SYSTEM);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_PULSER_SYSTEM);
    triggerHeader.setSource(TRIGSRC_BUFFNODE_SYSTEM);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_BUFFNODE_SYSTEM);
    triggerHeader.setSource(TRIGSRC_SIMULATED);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_SIMULATED);
    triggerHeader.setSource(TRIGSRC_USERDEF);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_USERDEF);
    triggerHeader.setSource(TRIGSRC_UNKNOWN);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_UNKNOWN);
    triggerHeader.setSource(TRIGSRC_DUMMY);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_DUMMY);

    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_1);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_1);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_2);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_2);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_3);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_3);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_4);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_4);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_5);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_5);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_6);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_6);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_7);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_7);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_8);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_8);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_9);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_9);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_10);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_10);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_11);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_11);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_12);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_12);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_13);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_13);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_14);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_14);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_15);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_15);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_16);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_16);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_17);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_17);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_18);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_18);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_19);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_19);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_20);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_20);
    triggerHeader.setSourceID(TRIGSRC_ID_BEAMSTAMP_NUMI_1);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BEAMSTAMP_NUMI_1);
    triggerHeader.setSourceID(TRIGSRC_ID_BEAMSTAMP_NUMI_2);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BEAMSTAMP_NUMI_2);
    triggerHeader.setSourceID(TRIGSRC_ID_BEAMSTAMP_BOOSTER_1);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BEAMSTAMP_BOOSTER_1);
    triggerHeader.setSourceID(TRIGSRC_ID_BEAMSTAMP_BOOSTER_2);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BEAMSTAMP_BOOSTER_2);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P1);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P1);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P2);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P2);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P3);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P3);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P4);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P4);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P5);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P5);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P6);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P6);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P7);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P7);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P8);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P8);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P9);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P9);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P10);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P10);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P11);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P11);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P12);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P12);
    triggerHeader.setSourceID(TRIGSRC_ID_DUMMY);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_DUMMY);

    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C1);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C1);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C2);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C2);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C3);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C3);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C4);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C4);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C5);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C5);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C6);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C6);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C7);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C7);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C8);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C8);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_DUMMY);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_DUMMY);

    for (uint32_t idx = 0; idx < 0x1000; ++idx) {
        triggerHeader.setVersion(idx);
        CPPUNIT_ASSERT(triggerHeader.getVersion() == idx);
    }

    triggerHeader.setTriggerNumberLow(PATTERN1);
    triggerHeader.setTriggerNumberHigh(PATTERN2);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberLow() == PATTERN1);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberHigh() ==
                   (PATTERN2 & 0x000000ff));

    uint64_t tmpValue = (PATTERN7 & 0x000000ff);
    uint64_t fullValue1 = PATTERN8 | (tmpValue << 32);
    triggerHeader.setTriggerNumber(fullValue1);

    CPPUNIT_ASSERT(triggerHeader.getTriggerNumber() == fullValue1);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberLow() == PATTERN8);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberHigh() ==
                   (PATTERN7 & 0x000000ff));

    // the delimiter is a little harder to test since it doesn't have
    // a "get" method
    triggerHeader.setDelimiter(PATTERN7 & 0x0000ffff);
    // limit scope of access to the header internal pointer
    {
        uint32_t* bufPtr = (uint32_t*) triggerHeader.getBuffer();
        CPPUNIT_ASSERT((SWAB32(bufPtr[0]) & 0xffff0000) >> 16 ==
                       (PATTERN7 & 0x0000ffff));
    }

    // check for cross-talk
    triggerHeader.setHWFlag(1);
    triggerHeader.setSource(TRIGSRC_USERDEF);
    triggerHeader.setSourceID(TRIGSRC_ID_BEAMSTAMP_BOOSTER_2);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C6);
    triggerHeader.setVersion(42);
    triggerHeader.setTriggerNumberLow(PATTERN9);
    triggerHeader.setTriggerNumberHigh(PATTERN0 & 0x000000ff);
    triggerHeader.setDelimiter(PATTERN4 & 0x0000ffff);

    CPPUNIT_ASSERT(triggerHeader.getHWFlag() == 1);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_USERDEF);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BEAMSTAMP_BOOSTER_2);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C6);
    CPPUNIT_ASSERT(triggerHeader.getVersion() == 42);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberLow() == PATTERN9);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberHigh() ==
                   (PATTERN0 & 0x000000ff));
}

void
RawTriggerHeaderTests::testSetAndGetWithExternalBuffer()
{
    // create a trigger header object based on an external buffer
    uint32_t buffer[BUFFER_WORDS];
    memset(buffer, 0, BUFFER_WORDS * sizeof(uint32_t));
    RawTriggerHeader triggerHeader((void*) buffer, BUFFER_WORDS);

    // test that we get the same values as what we set
    triggerHeader.setHWFlag(0);
    CPPUNIT_ASSERT(triggerHeader.getHWFlag() == 0);
    triggerHeader.setHWFlag(1);
    CPPUNIT_ASSERT(triggerHeader.getHWFlag() == 1);

    triggerHeader.setSource(TRIGSRC_BEAMSTAMP_SYSTEM);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_BEAMSTAMP_SYSTEM);
    triggerHeader.setSource(TRIGSRC_PULSER_SYSTEM);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_PULSER_SYSTEM);
    triggerHeader.setSource(TRIGSRC_BUFFNODE_SYSTEM);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_BUFFNODE_SYSTEM);
    triggerHeader.setSource(TRIGSRC_SIMULATED);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_SIMULATED);
    triggerHeader.setSource(TRIGSRC_USERDEF);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_USERDEF);
    triggerHeader.setSource(TRIGSRC_UNKNOWN);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_UNKNOWN);
    triggerHeader.setSource(TRIGSRC_DUMMY);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_DUMMY);

    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_1);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_1);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_2);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_2);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_3);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_3);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_4);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_4);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_5);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_5);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_6);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_6);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_7);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_7);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_8);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_8);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_9);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_9);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_10);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_10);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_11);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_11);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_12);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_12);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_13);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_13);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_14);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_14);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_15);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_15);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_16);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_16);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_17);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_17);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_18);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_18);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_19);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_19);
    triggerHeader.setSourceID(TRIGSRC_ID_BUFFNODE_20);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BUFFNODE_20);
    triggerHeader.setSourceID(TRIGSRC_ID_BEAMSTAMP_NUMI_1);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BEAMSTAMP_NUMI_1);
    triggerHeader.setSourceID(TRIGSRC_ID_BEAMSTAMP_NUMI_2);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BEAMSTAMP_NUMI_2);
    triggerHeader.setSourceID(TRIGSRC_ID_BEAMSTAMP_BOOSTER_1);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BEAMSTAMP_BOOSTER_1);
    triggerHeader.setSourceID(TRIGSRC_ID_BEAMSTAMP_BOOSTER_2);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BEAMSTAMP_BOOSTER_2);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P1);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P1);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P2);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P2);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P3);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P3);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P4);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P4);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P5);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P5);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P6);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P6);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P7);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P7);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P8);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P8);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P9);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P9);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P10);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P10);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P11);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P11);
    triggerHeader.setSourceID(TRIGSRC_ID_PULSER_P12);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_PULSER_P12);
    triggerHeader.setSourceID(TRIGSRC_ID_DUMMY);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_DUMMY);

    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C1);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C1);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C2);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C2);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C3);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C3);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C4);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C4);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C5);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C5);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C6);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C6);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C7);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C7);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C8);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C8);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_DUMMY);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_DUMMY);

    for (uint32_t idx = 0; idx < 0x1000; ++idx) {
        triggerHeader.setVersion(idx);
        CPPUNIT_ASSERT(triggerHeader.getVersion() == idx);
    }

    triggerHeader.setTriggerNumberLow(PATTERN1);
    triggerHeader.setTriggerNumberHigh(PATTERN2);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberLow() == PATTERN1);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberHigh() ==
                   (PATTERN2 & 0x000000ff));

    uint64_t tmpValue = (PATTERN7 & 0x000000ff);
    uint64_t fullValue1 = PATTERN8 | (tmpValue << 32);
    triggerHeader.setTriggerNumber(fullValue1);

    CPPUNIT_ASSERT(triggerHeader.getTriggerNumber() == fullValue1);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberLow() == PATTERN8);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberHigh() ==
                   (PATTERN7 & 0x000000ff));

    // the delimiter is a little harder to test since it doesn't have
    // a "get" method
    triggerHeader.setDelimiter(PATTERN7 & 0x0000ffff);
    // limit scope of access to the header internal pointer
    {
        uint32_t* bufPtr = (uint32_t*) triggerHeader.getBuffer();
        CPPUNIT_ASSERT((SWAB32(bufPtr[0]) & 0xffff0000) >> 16 ==
                       (PATTERN7 & 0x0000ffff));
    }

    // check for cross-talk
    triggerHeader.setHWFlag(1);
    triggerHeader.setSource(TRIGSRC_USERDEF);
    triggerHeader.setSourceID(TRIGSRC_ID_BEAMSTAMP_BOOSTER_2);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C6);
    triggerHeader.setVersion(42);
    triggerHeader.setTriggerNumberLow(PATTERN9);
    triggerHeader.setTriggerNumberHigh(PATTERN0 & 0x000000ff);
    triggerHeader.setDelimiter(PATTERN4 & 0x0000ffff);

    CPPUNIT_ASSERT(triggerHeader.getHWFlag() == 1);
    CPPUNIT_ASSERT(triggerHeader.getSource() == TRIGSRC_USERDEF);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_BEAMSTAMP_BOOSTER_2);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == TRIGSRC_SUBID_PULSER_C6);
    CPPUNIT_ASSERT(triggerHeader.getVersion() == 42);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberLow() == PATTERN9);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberHigh() ==
                   (PATTERN0 & 0x000000ff));

    // verify that we get the expected results after we clear the buffer
    memset(buffer, 0, BUFFER_WORDS * sizeof(uint32_t));
    CPPUNIT_ASSERT(triggerHeader.getHWFlag() == 0);
    CPPUNIT_ASSERT(triggerHeader.getSource() == 0);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == 0);
    CPPUNIT_ASSERT(triggerHeader.getSourceSubID() == 0);
    CPPUNIT_ASSERT(triggerHeader.getVersion() == 0);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberLow() == 0);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberHigh() == 0);
}

void
RawTriggerHeaderTests::testCopyConstructorWithReference()
{
    RawTriggerHeader triggerHeader;

    // set values in the first trigger header object
    triggerHeader.setHWFlag(1);
    triggerHeader.setSource(TRIGSRC_USERDEF);
    triggerHeader.setSourceID(TRIGSRC_ID_BEAMSTAMP_BOOSTER_1);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C5);
    triggerHeader.setVersion(33);
    triggerHeader.setTriggerNumberLow(PATTERN9);
    triggerHeader.setTriggerNumberHigh(PATTERN0 & 0x000000ff);
    triggerHeader.setDelimiter(PATTERN4 & 0x0000ffff);

    // create a second trigger header object from the first
    RawTriggerHeader headerCopy(triggerHeader);

    // reset the values in the first instance
    triggerHeader.setHWFlag(0);
    triggerHeader.setSource(TRIGSRC_DUMMY);
    triggerHeader.setSourceID(TRIGSRC_ID_DUMMY);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_DUMMY);
    triggerHeader.setVersion(0);
    triggerHeader.setTriggerNumberLow(0);
    triggerHeader.setTriggerNumberHigh(0);
    triggerHeader.setDelimiter(0);

    // verify that the copy has the original values
    CPPUNIT_ASSERT(headerCopy.getHWFlag() == 1);
    CPPUNIT_ASSERT(headerCopy.getSource() == TRIGSRC_USERDEF);
    CPPUNIT_ASSERT(headerCopy.getSourceID() ==
                   TRIGSRC_ID_BEAMSTAMP_BOOSTER_1);
    CPPUNIT_ASSERT(headerCopy.getSourceSubID() == TRIGSRC_SUBID_PULSER_C5);
    CPPUNIT_ASSERT(headerCopy.getVersion() == 33);
    CPPUNIT_ASSERT(headerCopy.getTriggerNumberLow() == PATTERN9);
    CPPUNIT_ASSERT(headerCopy.getTriggerNumberHigh() ==
                   (PATTERN0 & 0x000000ff));
}

void
RawTriggerHeaderTests::testConstructionFromExternalBuffer()
{
    // create a buffer with sample trigger header data
    uint32_t buffer[BUFFER_WORDS];
    memset(buffer, 0, BUFFER_WORDS * sizeof(uint32_t));
    buffer[0] |= SWAB32(PATTERN1 & 0x00000fff);
    buffer[1] |= SWAB32(0x80000000);
    buffer[1] |= SWAB32(0x0000ffff);
    buffer[2] |= SWAB32(PATTERN2);
    buffer[3] |= SWAB32(PATTERN3 & 0x000000ff);

    // create the the trigger header object from the buffer
    RawTriggerHeader triggerHeader((void*) &buffer[0]);

    // verify that the copy has the original values
    CPPUNIT_ASSERT(triggerHeader.getHWFlag() == 1);
    CPPUNIT_ASSERT(triggerHeader.getSourceID() == TRIGSRC_ID_DUMMY);
    CPPUNIT_ASSERT(triggerHeader.getVersion() == (PATTERN1 & 0x00000fff));
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberLow() == PATTERN2);
    CPPUNIT_ASSERT(triggerHeader.getTriggerNumberHigh() ==
                   (PATTERN3 & 0x000000ff));
}

void
RawTriggerHeaderTests::testNoDeleteOfExternalBuffer()
{
    uint32_t* buffer = new uint32_t[BUFFER_WORDS];
    memset(buffer, 0, BUFFER_WORDS * sizeof(uint32_t));

    // create a trigger header object based on the external buffer
    {
        RawTriggerHeader triggerHeader((void*) buffer, BUFFER_WORDS);
        triggerHeader.setHWFlag(1);
        triggerHeader.setSourceID(TRIGSRC_ID_DUMMY);
        triggerHeader.setVersion(PATTERN1 & 0x00000fff);
        triggerHeader.setTriggerNumberLow(PATTERN2);
        triggerHeader.setTriggerNumberHigh(PATTERN3 & 0x000000ff);
        triggerHeader.setDelimiter(PATTERN4 & 0x0000ffff);
    }

    // verify that the buffer is still valid after the header object
    // goes out of scope and is deleted
    CPPUNIT_ASSERT(buffer[0] ==
                   SWAB32(((PATTERN4 & 0x0000ffff) << 16) |
                        (PATTERN1 & 0x00000fff)));
    CPPUNIT_ASSERT(buffer[1] == SWAB32(0x8000ffff));
    CPPUNIT_ASSERT(buffer[2] == SWAB32(PATTERN2));
    CPPUNIT_ASSERT(buffer[3] == SWAB32((PATTERN3 & 0x000000ff)));

    delete[] buffer;
}

void
RawTriggerHeaderTests::testNoModsToExternalBuffer()
{
    uint32_t* buffer = new uint32_t[BUFFER_WORDS];

    // initialize the buffer to all zeroes
    memset(buffer, 0, BUFFER_WORDS * sizeof(uint32_t));

    // create a trigger header object based on the external buffer
    {
        RawTriggerHeader triggerHeader((void*) buffer, BUFFER_WORDS);
    }

    // verify that the buffer is unchanged from its use by the header object
    for (uint32_t idx = 0; idx < BUFFER_WORDS; ++idx) {
        CPPUNIT_ASSERT(buffer[idx] == 0);
    }

    // initialize the buffer to all ones
    memset(buffer, 0xff, BUFFER_WORDS * sizeof(uint32_t));

    // create another trigger header object based on the external buffer
    {
        RawTriggerHeader triggerHeader((void*) buffer, BUFFER_WORDS);
    }

    // verify that the buffer is unchanged from its use by the header object
    for (uint32_t idx = 0; idx < BUFFER_WORDS; ++idx) {
        CPPUNIT_ASSERT(buffer[idx] == 0xffffffff);
    }

    delete[] buffer;
}

void
RawTriggerHeaderTests::testSmallExternalBuffer()
{
    // create a buffer that it too small to hold header information
    uint32_t buffer[SMALL_BUFFER_WORDS];
    memset(buffer, 0, SMALL_BUFFER_WORDS * sizeof(uint32_t));

    // create a header object and expect an exception because the
    // buffer is too small
    try {
        RawTriggerHeader triggerHeader((void*) buffer, SMALL_BUFFER_WORDS);

        // if we get to the following assert, something failed (there
        // should have been an exception thrown
        CPPUNIT_ASSERT(0);
    }
    // an exception is the expected behavior, so we report success
    catch (DataFormatException& dfExcept) {
        CPPUNIT_ASSERT(1);
    }
}

void
RawTriggerHeaderTests::testExtBuffToExtBuffCopy()
{
    // create two trigger header objects, both based on external buffers
    uint32_t buffer1[BUFFER_WORDS];
    uint32_t buffer2[BUFFER_WORDS];
    memset(buffer1, 0, BUFFER_WORDS * sizeof(uint32_t));
    memset(buffer2, 0, BUFFER_WORDS * sizeof(uint32_t));
    RawTriggerHeader triggerHeader((void*) buffer1, BUFFER_WORDS);
    RawTriggerHeader headerCopy((void*) buffer2, BUFFER_WORDS);

    // set values in the first trigger header object
    triggerHeader.setHWFlag(1);
    triggerHeader.setSource(TRIGSRC_USERDEF);
    triggerHeader.setSourceID(TRIGSRC_ID_BEAMSTAMP_BOOSTER_1);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_PULSER_C5);
    triggerHeader.setVersion(33);
    triggerHeader.setTriggerNumberLow(PATTERN9);
    triggerHeader.setTriggerNumberHigh(PATTERN0 & 0x000000ff);
    triggerHeader.setDelimiter(PATTERN4 & 0x0000ffff);

    // copy the data from the first object into the second
    headerCopy.copy(triggerHeader);

    // reset the values in the first instance
    triggerHeader.setHWFlag(0);
    triggerHeader.setSource(TRIGSRC_DUMMY);
    triggerHeader.setSourceID(TRIGSRC_ID_DUMMY);
    triggerHeader.setSourceSubID(TRIGSRC_SUBID_DUMMY);
    triggerHeader.setVersion(0);
    triggerHeader.setTriggerNumberLow(0);
    triggerHeader.setTriggerNumberHigh(0);
    triggerHeader.setDelimiter(0);

    // verify that the copy has the original values
    CPPUNIT_ASSERT(headerCopy.getHWFlag() == 1);
    CPPUNIT_ASSERT(headerCopy.getSource() == TRIGSRC_USERDEF);
    CPPUNIT_ASSERT(headerCopy.getSourceID() ==
                   TRIGSRC_ID_BEAMSTAMP_BOOSTER_1);
    CPPUNIT_ASSERT(headerCopy.getSourceSubID() == TRIGSRC_SUBID_PULSER_C5);
    CPPUNIT_ASSERT(headerCopy.getVersion() == 33);
    CPPUNIT_ASSERT(headerCopy.getTriggerNumberLow() == PATTERN9);
    CPPUNIT_ASSERT(headerCopy.getTriggerNumberHigh() ==
                   (PATTERN0 & 0x000000ff));
}

void
RawTriggerHeaderTests::testInitializationOfDelimiter()
{
    RawTriggerHeader triggerHeader;
    uint32_t* bufPtr = (uint32_t*) triggerHeader.getBuffer();
    CPPUNIT_ASSERT((SWAB32(bufPtr[0]) & 0xffff0000) >> 16 ==
                   TriggerHeaderDelimiter);
}

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(RawTriggerHeaderTests, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");
